import 'package:flutter/material.dart';
import 'package:hello_flutter/ezyclient/ezy_logger.dart';

import 'ezyclient/ezy_constants.dart';
import 'ezyclient/ezy_entities.dart';
import 'ezyclient/ezy_handlers.dart';
import 'ezyclient/ezy_clients.dart';
import 'ezyclient/ezy_client.dart';
import 'ezyclient/ezy_config.dart';

const ZONE_NAME = "example";
const APP_NAME = "hello-world";

void main() {
  runApp(const MyApp());
}

class MyApp extends StatelessWidget {
  const MyApp({Key? key}) : super(key: key);

  // This widget is the root of your application.
  @override
  Widget build(BuildContext context) {
    return MaterialApp(
      title: 'Flutter Demo',
      theme: ThemeData(
        // This is the theme of your application.
        //
        // Try running your application with "flutter run". You'll see the
        // application has a blue toolbar. Then, without quitting the app, try
        // changing the primarySwatch below to Colors.green and then invoke
        // "hot reload" (press "r" in the console where you ran "flutter run",
        // or simply save your changes to "hot reload" in a Flutter IDE).
        // Notice that the counter didn't reset back to zero; the application
        // is not restarted.
        primarySwatch: Colors.blue,
      ),
      home: const MyHomePage(title: 'Flutter Demo Home Page'),
    );
  }
}

class MyHomePage extends StatefulWidget {
  const MyHomePage({Key? key, required this.title}) : super(key: key);

  // This widget is the home page of your application. It is stateful, meaning
  // that it has a State object (defined below) that contains fields that affect
  // how it looks.

  // This class is the configuration for the state. It holds the values (in this
  // case the title) provided by the parent (in this case the App widget) and
  // used by the build method of the State. Fields in a Widget subclass are
  // always marked "final".

  final String title;

  @override
  State<MyHomePage> createState() => _MyHomePageState();
}

class _MyHomePageState extends State<MyHomePage> {
  int _counter = 0;

  String socketState = 'Socket has not connected yet';
  String sslMessage = '';

  Future<void> connectToServer() async {
    EzyConfig config = EzyConfig();
    config.clientName = "hello";
    config.enableSSL = true;
    config.enableDebug = true;
    EzyClients clients = EzyClients.getInstance();
    EzyClient client = clients.newDefaultClient(config);
    client.setup.addDataHandler(EzyCommand.HANDSHAKE, _SocketHandshakeHandler());
    client.setup.addDataHandler(EzyCommand.LOGIN, _SocketLoginSuccessHandler());
    client.setup.addDataHandler(EzyCommand.APP_ACCESS, _SocketAppAccessHandler());
    var appSetup = client.setup.setupApp(APP_NAME);
    appSetup.addDataHandler("greet", _SocketGreetResponseHandler((message) {
      setState(() {
        socketState = message;
      });
    }));
    appSetup.addDataHandler("secureChat", _SocketSecureChatResponseHandler((message) {
      setState(() {
        sslMessage = message;
      });
    }));
    client.connect("tvd12.com", 3005);
    // client.connect("127.0.0.1", 3005);
  }

  void _incrementCounter() {
    setState(() {
      // This call to setState tells the Flutter framework that something has
      // changed in this State, which causes it to rerun the build method below
      // so that the display can reflect the updated values. If we changed
      // _counter without calling setState(), then the build method would not be
      // called again, and so nothing would appear to happen.
      _counter++;
    });

    connectToServer();
  }

  @override
  Widget build(BuildContext context) {
    // This method is rerun every time setState is called, for instance as done
    // by the _incrementCounter method above.
    //
    // The Flutter framework has been optimized to make rerunning build methods
    // fast, so that you can just rebuild anything that needs updating rather
    // than having to individually change instances of widgets.
    return Scaffold(
      appBar: AppBar(
        // Here we take the value from the MyHomePage object that was created by
        // the App.build method, and use it to set our appbar title.
        title: Text(widget.title),
      ),
      body: Center(
        // Center is a layout widget. It takes a single child and positions it
        // in the middle of the parent.
        child: Column(
          // Column is also a layout widget. It takes a list of children and
          // arranges them vertically. By default, it sizes itself to fit its
          // children horizontally, and tries to be as tall as its parent.
          //
          // Invoke "debug painting" (press "p" in the console, choose the
          // "Toggle Debug Paint" action from the Flutter Inspector in Android
          // Studio, or the "Toggle Debug Paint" command in Visual Studio Code)
          // to see the wireframe for each widget.
          //
          // Column has various properties to control how it sizes itself and
          // how it positions its children. Here we use mainAxisAlignment to
          // center the children vertically; the main axis here is the vertical
          // axis because Columns are vertical (the cross axis would be
          // horizontal).
          mainAxisAlignment: MainAxisAlignment.center,
          children: <Widget>[
            const Text(
              'You have pushed the button this many times:',
            ),
            Text(
              '$_counter',
              style: Theme.of(context).textTheme.headline4,
            ),
            const Text(
              'Socket message: ',
            ),
            Text(
              '$socketState',
              style: Theme.of(context).textTheme.headline6,
            ),
            const Text(
              'SSL message: ',
            ),
            Text(
              '$sslMessage',
              style: Theme.of(context).textTheme.headline6,
            )
          ],
        ),
      ),
      floatingActionButton: FloatingActionButton(
        onPressed: _incrementCounter,
        tooltip: 'Increment',
        child: const Icon(Icons.add),
      ), // This trailing comma makes auto-formatting nicer for build methods.
    );
  }
}

class _SocketHandshakeHandler extends EzyHandshakeHandler {
  @override
  List getLoginRequest() {
    var request = [];
    request.add("example");
    request.add("username");
    request.add("password");
    request.add([]);
    return request;
  }
}

class _SocketLoginSuccessHandler extends EzyLoginSuccessHandler {
  @override
  void handleLoginSuccess(responseData) {
    client.send(EzyCommand.APP_ACCESS, [APP_NAME]);
  }
}

class _SocketAppAccessHandler extends EzyAppAccessHandler {
  @override
  void postHandle(EzyApp app, List data) {
    app.send("greet", {"who": "Flutter's developer"});
  }
}

class _SocketGreetResponseHandler extends EzyAppDataHandler<Map> {

  late Function(String) _callback;

  _SocketGreetResponseHandler(Function(String) callback) {
    _callback = callback;
  }

  @override
  void handle(EzyApp app, Map data) {
    _callback(data["message"]);
    app.send("secureChat", {"who": "Young Monkey"}, true);
  }
}

class _SocketSecureChatResponseHandler extends EzyAppDataHandler<Map> {

  late Function(String) _callback;

  _SocketSecureChatResponseHandler(Function(String) callback) {
    _callback = callback;
  }

  @override
  void handle(EzyApp app, Map data) {
    _callback(data["secure-message"]);
  }
}